/// @addtogroup auxil
/// @{

/////////////////////////////////////////////////////////////////////////////
/// @file Stopwatch.h
///
/// @author Marian Mrozek
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 2000-2006 by the CAPD Group.
//
// This file is part of the CAPD library.  This library is free software;
// you can redistribute it and/or modify it under the terms of the GNU
// General Public License as published by the Free Software Foundation;
// either version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this software; see the file "license.txt".  If not, write to the
// Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
// MA 02111-1307, USA.

#ifndef _STOPWATCH_H_
#define _STOPWATCH_H_

#include "auxil/clock.h"

class Stopwatch{
  private:
    long double startWorldSecs,startProcessSecs;
  public:
    Stopwatch():
      startWorldSecs(getWorldSeconds()),
      startProcessSecs(getProcessSeconds()){
    }
    double worldTime() const{
      return (double)(getWorldSeconds()-startWorldSecs);
    }
    double processTime() const{
      return (double)(getProcessSeconds()-startProcessSecs);
    }
    friend std::ostream& operator<<(std::ostream& out, const Stopwatch& sw){
      double wt=sw.worldTime();
      double pt= sw.processTime();
      out   << pt << " proc secs (" << wt << " world secs)";
      return out;
    }
};


#endif //  _STOPWATCH_H_
/// @}

