/// @addtogroup bitSet
/// @{

/////////////////////////////////////////////////////////////////////////////
/// @file EuclBitSetPixelNeighborOffset.h
///
/// @author Marian Mrozek
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 2000-2006 by the CAPD Group.
//
// This file is part of the CAPD library.  This library is free software;
// you can redistribute it and/or modify it under the terms of the GNU
// General Public License as published by the Free Software Foundation;
// either version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this software; see the file "license.txt".  If not, write to the
// Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
// MA 02111-1307, USA.

#if !defined(_BMPCUBSETPIXELNEIGHBOROFFSET_H_)
#define _BMPCUBSETPIXELNEIGHBOROFFSET_H_

template <typename P_BitSet,int dim>
struct EuclBitSetT<P_BitSet,dim>::PixelNeighborOffset{
  int offset[dim];
  static PixelNeighborOffset bottom(){
    PixelNeighborOffset o;
    for(int i=0;i<dim;++i) o.offset[i]=-1;
    return o;
  }
  static PixelNeighborOffset top(){
    PixelNeighborOffset o;
    for(int i=0;i<dim;++i) o.offset[i]=1;
    return o;
  }
  bool operator==(const PixelNeighborOffset& o2) const{
    for(int i=0;i<dim;++i) if(offset[i]!=o2.offset[i]) return false;
    return true;
  }
  PixelNeighborOffset& operator++(){
    for(int i=0;i<dim;++i){
      ++offset[i];
      if(offset[i]<=1) break;
      else offset[i]=-1;
    }
    return *this;
  }
};
#endif //_BMPCUBSETPIXELNEIGHBOROFFSET_H_



/// @}

