/// @addtogroup covrel
/// @{

/////////////////////////////////////////////////////////////////////////////
/// @file hsetNDIntf.h
///
/// @author Daniel Wilczak
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 2000-2005 by the CAPD Group.
//
// This file is part of the CAPD library.  This library is free software;
// you can redistribute it and/or modify it under the terms of the GNU
// General Public License as published by the Free Software Foundation;
// either version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this software; see the file "license.txt".  If not, write to the
// Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
// MA 02111-1307, USA.

#ifndef _HSetNDIntf
#define _HSetNDIntf

#include "covrel/hset.h"
#include "dynset/c0set.h"

/**
   This class provides a h-set in an arbitrary dimension
*/
namespace capd{
namespace covrel{

template<typename VectorType, typename IVectorType, typename MatrixType, typename IMatrixType>
class hSetND : public hSet <VectorType,IVectorType> {
public:
   hSetND(){}
   // we assume that the set is represented as A_Center + A_M *B(0,1)
   // d1 denotes a number of unstable directions
   // d2 denotes a number of stable directions
   hSetND(const VectorType& A_Center, const MatrixType& A_M, int A_d1, int A_d2);
   hSetND(const IVectorType &A_Center, const IMatrixType &A_M, int A_d1, int A_d2);
   ~hSetND(){}

// double versions
   bool outside(const VectorType &point) const;
   bool inside (const VectorType &point) const;
   bool across (const VectorType &point) const;
   bool mapaway(const VectorType &point) const;

// interval versions
   bool outside(const IVectorType &point) const;
   bool inside (const IVectorType &point) const;
   bool across (const IVectorType &point) const;
   bool mapaway(const IVectorType &point) const;

// if we have a special represenation of a point
   bool outside(const capd::dynset::C0set<IMatrixType>& A_set) const;
   bool inside (const capd::dynset::C0set<IMatrixType>& A_set) const;
   bool across (const capd::dynset::C0set<IMatrixType>& A_set) const;
   bool mapaway(const capd::dynset::C0set<IMatrixType>& A_set) const;

protected:
   bool checkOutside(const IVectorType&) const;
   bool checkInside (const IVectorType&) const;
   bool checkAcross (const IVectorType&) const;
   bool checkMapaway(const IVectorType&) const;


   VectorType m_C; // center point
   MatrixType m_M; // coordinate system
   IVectorType m_IC;
   IMatrixType m_IM;

   MatrixType m_invM; // iverse matrix to m_M
   IMatrixType m_invIM; // iverse matrix to m_IM

   int m_d1, m_d2;
};

// sometimes we need a pointer to a function which
// verifies covering relations for h-sets
//    int (*f)(hSetND &, IntevalVector &);
// the functions are:
/*
bool OutSide  (const hSetND&, const hSet3D::VectorType &);
bool InSide   (const hSetND&, const hSet3D::VectorType &);
bool Across   (const hSetND&, const hSet3D::VectorType &);
bool MapAway  (const hSetND&, const hSet3D::VectorType &);
bool LeftSide (const hSetND&, const hSet3D::VectorType &);
bool RightSide(const hSetND&, const hSet3D::VectorType &);

bool OutSide  (const hSet3D&, const hSet3D::IVectorType &);
bool InSide   (const hSet3D&, const hSet3D::IVectorType &);
bool Across   (const hSet3D&, const hSet3D::IVectorType &);
bool MapAway  (const hSet3D&, const hSet3D::IVectorType &);
bool LeftSide (const hSet3D&, const hSet3D::IVectorType &);
bool RightSide(const hSet3D&, const hSet3D::IVectorType &);
*/
}} // namespace capd::covrel

#endif // _HSetNDIntf

/// @}
