/// @addtogroup diffIncl
/// @{

/////////////////////////////////////////////////////////////////////////////
/// @file DiffInclusion.hpp
///
/// @author Tomasz Kapela
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 2000-2005 by the CAPD Group.
//
// This file is part of the CAPD library.  This library is free software;
// you can redistribute it and/or modify it under the terms of the GNU
// General Public License as published by the Free Software Foundation;
// either version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this software; see the file "license.txt".  If not, write to the
// Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
// MA 02111-1307, USA.

/* Author: Tomasz Kapela, 2007 */

#ifndef _DIFF_INCLUSION_HPP_
#define _DIFF_INCLUSION_HPP_

#include <sstream>
#include <string>
#include <stdexcept>

#include "dynsys/taylor.h"

namespace capd{
namespace diffIncl{


template<typename MapT>
DiffInclusion<MapT>::DiffInclusion(
           MultiMapType& A_diffIncl, 
           int A_order, 
           const ScalarType& A_step, 
           const NormType & norm
) 
  : capd::dynsys::BasicTaylor<MapType>(A_diffIncl.getVectorField(), A_order, A_step), 
    capd::dynsys::Taylor<MapType>(A_diffIncl.getVectorField(), A_order, A_step), 
    m_norm ( norm.clone()),
    m_diffIncl(A_diffIncl) {
}

template<typename MapT>
DiffInclusion<MapT>::~DiffInclusion(){
  delete m_norm;
}
  
/////////////////////////////////////////////////////////////////////////////
// DiffInclusion::inclusionEnclosure
///
/// Computes enclosure of image of given set for differential inclusion during whole time step
///
/// It requires that parameters of the function are intervals that include all perturbations  
///
/////////////////////////////////////////////////////////////////////////////
template<typename MapT>
inline typename DiffInclusion<MapT>::VectorType DiffInclusion<MapT>::diffInclusionEnclosure(const VectorType& x) const{

  return capd::dynsys::enclosure(m_diffIncl, x, m_step);
}

  
}} //namespace capd::diffIncl`

#endif // _DIFF_INCLUSION_HPP_ 

/// @}
