/// @addtogroup diffIncl
/// @{

/////////////////////////////////////////////////////////////////////////////
/// @file DiffInclusionLN.hpp
///
/// @author Tomasz Kapela
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 2000-2005 by the CAPD Group.
//
// This file is part of the CAPD library.  This library is free software;
// you can redistribute it and/or modify it under the terms of the GNU
// General Public License as published by the Free Software Foundation;
// either version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this software; see the file "license.txt".  If not, write to the
// Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
// MA 02111-1307, USA.

/* Author: Tomasz Kapela, 2007 */

#ifndef _DIFF_INCLUSION_LN_HPP_
#define _DIFF_INCLUSION_LN_HPP_

#include "diffIncl/diffInclusionLNIntf.h"

namespace capd{
namespace diffIncl{


template<typename MapT>
DiffInclusionLN<MapT>::DiffInclusionLN(
           MultiMapType& A_diffIncl, 
           int A_order, 
           ScalarType const & A_step, 
           NormType const & A_norm
) 
  : capd::dynsys::BasicTaylor<MapType>(A_diffIncl.getVectorField(), A_order, A_step), 
    capd::diffIncl::DiffInclusion<MapT>(A_diffIncl, A_order, A_step, A_norm) {
}

template<typename MapT>
typename DiffInclusionLN<MapT>::VectorType DiffInclusionLN<MapT>::perturbations(const VectorType& x){
  
  int OKflag;
  VectorType W_1 = enclosure(x, &OKflag);
  VectorType W_2 = diffInclusionEnclosure(x);
  
  MatrixType J = m_diffIncl[W_2]; 
  VectorType deltha = m_diffIncl.perturbations(W_1);
 
  ScalarType C = right((*m_norm)(deltha));
  ScalarType l = right((*m_norm)(J));
  
  
 
  ScalarType D = (l.contains(0.0))? C*getStep() : (C*(exp(l*getStep())-1))/l;
  VectorType result(x.dimension());
  
  for(int i=0; i< result.dimension(); ++i)
    result[i] = ScalarType(-D.rightBound(), D.rightBound());
  
  return result;
}
  
}} //namespace capd::diffIncl

#endif // 

/// @}
