/// @addtogroup diffIncl
/// @{

/////////////////////////////////////////////////////////////////////////////
/// @file DiffInclusionLNIntf.h
///
/// @author Tomasz Kapela
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 2000-2005 by the CAPD Group.
//
// This file is part of the CAPD library.  This library is free software;
// you can redistribute it and/or modify it under the terms of the GNU
// General Public License as published by the Free Software Foundation;
// either version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this software; see the file "license.txt".  If not, write to the
// Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
// MA 02111-1307, USA.

/* Author: Tomasz Kapela, 2007 */

#ifndef _DIFF_INCLUSION_LN_INTF_H_
#define _DIFF_INCLUSION_LN_INTF_H_

#include "diffIncl/diffInclusionIntf.h"

namespace capd{
namespace diffIncl{

template<typename MapT>
class DiffInclusionLN : public capd::diffIncl::DiffInclusion<MapT>{

public:
  typedef typename DiffInclusion<MapT>::MultiMapType   MultiMapType;
  typedef typename DiffInclusion<MapT>::MapType        MapType;
  typedef typename DiffInclusion<MapT>::FunctionType   FunctionType;
  typedef typename DiffInclusion<MapT>::MatrixType     MatrixType;
  typedef typename DiffInclusion<MapT>::VectorType     VectorType;
  typedef typename DiffInclusion<MapT>::ScalarType     ScalarType;
  typedef typename DiffInclusion<MapT>::NormType       NormType;
  
  DiffInclusionLN( MultiMapType & diffIncl, 
                   int order, 
                   ScalarType const & step,
                   NormType const & norm
  );

  VectorType perturbations(const VectorType & x);

  using DiffInclusion<MapT>::enclosure;
  using DiffInclusion<MapT>::diffInclusionEnclosure;
  using DiffInclusion<MapT>::getStep;

protected:
  using DiffInclusion<MapT>::m_vField;
  using DiffInclusion<MapT>::m_order;
  using DiffInclusion<MapT>::m_step;
  using DiffInclusion<MapT>::m_dimension;
  using DiffInclusion<MapT>::m_norm;
  using DiffInclusion<MapT>::m_diffIncl;
  
};


}} // namespace capd::diffIncl

#endif //

/// @}
