/// @addtogroup dynset
/// @{

/////////////////////////////////////////////////////////////////////////////
/// @file intv2set.hpp
///
/// @author The CAPD Group
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 2000-2005 by the CAPD Group.
//
// This file is part of the CAPD library.  This library is free software;
// you can redistribute it and/or modify it under the terms of the GNU
// General Public License as published by the Free Software Foundation;
// either version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this software; see the file "license.txt".  If not, write to the
// Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
// MA 02111-1307, USA.

#ifndef _Intv2Set_HPP
#define _Intv2Set_HPP

#include "dynset/intv2setintf.h"

namespace capd{
namespace dynset{

template<typename MatrixType>
typename Intv2Set<MatrixType>::ScalarType Intv2Set<MatrixType>::size(void)
{
   return ::size( m_x + m_C*m_r0 + m_r );
}

template<typename MatrixType>
void Intv2Set<MatrixType>::move(capd::dynsys::DynSys<MatrixType>& dynsys)
{
   VectorType s(m_x.dimension());
   MatrixType S(m_x.dimension(),m_x.dimension());

   VectorType xx = m_x + m_C*m_r0 + m_r;

   VectorType y = dynsys.Phi(m_x);
   split(y,s);
   s = s+dynsys.Remainder(xx);
   MatrixType A = dynsys.JacPhi(xx);
   m_C = A*m_C;
   splitMatrix(m_C,S);
   m_x = y;
   m_r = s + S*m_r0 + A*m_r;
}

template<typename MatrixType>
void Intv2Set<MatrixType>::move(capd::dynsys::DynSys<MatrixType>& dynsys, Intv2Set& result) const
{
   VectorType s(m_x.dimension());
   MatrixType S(m_x.dimension(),m_x.dimension());

   VectorType xx = m_x + m_C*m_r0 + m_r;

   VectorType y = dynsys.Phi(m_x);
   split(y,s);
   s = s + dynsys.Remainder(xx);
   MatrixType A = dynsys.JacPhi(xx);
   result.m_C = A*m_C;
   splitMatrix(result.m_C,S);
   result.m_x = y;
   result.m_r = s + S*m_r0 + A*m_r;
   result.m_r0 = m_r0;
}

template<typename MatrixType>
std::string Intv2Set<MatrixType>::show(void) const
{
   std::ostringstream descriptor;
   descriptor << "IntervalSet: x=";
   descriptor << m_x << "\n r=";
   descriptor << m_r << "\n C=";
   descriptor << m_C << "\n r0=";
   descriptor << m_r0 << " ";
   return descriptor.str();
}

template<typename MatrixType>
typename Intv2Set<MatrixType>::VectorType Intv2Set<MatrixType>::affineTransformation(
      const MatrixType& A_M, const VectorType& A_C) const
{
   return A_M*(m_x+m_r-A_C) + (A_M*m_C)*m_r0;
}

}} // namespace capd::dynset

#endif

/// @}
