/// @addtogroup dynset
/// @{

/////////////////////////////////////////////////////////////////////////////
/// @file intvset.h
///
/// @author The CAPD Group
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 2000-2005 by the CAPD Group.
//
// This file is part of the CAPD library.  This library is free software;
// you can redistribute it and/or modify it under the terms of the GNU
// General Public License as published by the Free Software Foundation;
// either version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this software; see the file "license.txt".  If not, write to the
// Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
// MA 02111-1307, USA.

#ifndef _IntervalSetIntf_H
#define _IntervalSetIntf_H

#include "dynset/c0set.h"

// move  by  Jac*x + remainder
namespace capd{
namespace dynset{

template<typename MatrixT>
class IntervalSet : public C0set<MatrixT>
{
public:
   typedef MatrixT MatrixType;
   typedef typename MatrixType::RowVectorType VectorType;
   typedef typename MatrixType::ScalarType ScalarType;
   typedef Norm<VectorType,MatrixType> NormType;

   IntervalSet(void);
   explicit IntervalSet(int);
   explicit IntervalSet(const VectorType& the_x);
   IntervalSet(const VectorType& the_x,const VectorType& the_r);
   virtual C0set<MatrixType>* clone(void);
   virtual C0set<MatrixType>* fresh(void);
   virtual C0set<MatrixType>* cover(const VectorType& v);
   virtual ScalarType size(void);
   virtual VectorType center(void);
   void move(capd::dynsys::DynSys<MatrixType>& dynsys);
   void move(capd::dynsys::DynSys<MatrixType>& dynsys, IntervalSet &result) const;
   virtual std::string show(void) const;
   virtual operator VectorType() const;
   VectorType get_r();
   virtual VectorType affineTransformation(const MatrixType&, const VectorType&) const;

protected:
   VectorType m_x, m_r;
};

// inline definitions

template<typename MatrixType>
inline IntervalSet<MatrixType>::IntervalSet(void)
{
   m_x.clear(); m_r.clear();
}

template<typename MatrixType>
inline IntervalSet<MatrixType>::IntervalSet(int dim)
   : m_x(dim), m_r(dim)
{
   m_x.clear(); m_r.clear();
}

template<typename MatrixType>
inline typename IntervalSet<MatrixType>::VectorType IntervalSet<MatrixType>::center(void)
{
   return m_x;
}

template<typename MatrixT>
inline IntervalSet<MatrixT>::operator typename IntervalSet<MatrixT>::VectorType(void) const
{
   return m_x + m_r;
}

template<typename MatrixType>
inline IntervalSet<MatrixType>::IntervalSet(const VectorType& the_x)
   : m_x(the_x), m_r(the_x.dimension())
{
   m_r.clear();
}

template<typename MatrixType>
inline IntervalSet<MatrixType>::IntervalSet(const VectorType& the_x, const VectorType& the_r)
   : m_x(the_x), m_r(the_r)
{}

template<typename MatrixType>
inline C0set<MatrixType>* IntervalSet<MatrixType>::clone(void)
{
   return new IntervalSet<MatrixType>(*this);
}

template<typename MatrixType>
inline C0set<MatrixType>* IntervalSet<MatrixType>::fresh(void)
{
   return new IntervalSet<MatrixType>(m_x.dimension());
}

template<typename MatrixType>
inline C0set<MatrixType>* IntervalSet<MatrixType>::cover(const VectorType& v)
{
   return new IntervalSet<MatrixType>(midVector(v),v-midVector(v));
}

template<typename MatrixType>
inline typename IntervalSet<MatrixType>::VectorType IntervalSet<MatrixType>::get_r()
{
   return m_r;
}

}} // namespace capd::dynset

#endif

/// @}
