/// @addtogroup dynsys
/// @{

/////////////////////////////////////////////////////////////////////////////
/// @file taylorintf.h
///
/// @author Daniel Wilczak
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 2000-2005 by the CAPD Group.
//
// This file is part of the CAPD library.  This library is free software;
// you can redistribute it and/or modify it under the terms of the GNU
// General Public License as published by the Free Software Foundation;
// either version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this software; see the file "license.txt".  If not, write to the
// Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
// MA 02111-1307, USA.

/** Author: Daniel Wilczak, 2001-2005 */

#ifndef _TaylorClass
#define _TaylorClass

#include <string>
#include "vectalg/normintf.h"
#include "dynsys/c1dynsys.h"
#include "dynsys/basicTaylorIntf.h"

namespace capd{
namespace dynsys{

template<typename MapT>
class Taylor:   public virtual C1DynSys<typename MapT::MatrixType>, public virtual BasicTaylor<MapT>
{
public:
   typedef MapT MapType;
   typedef typename MapT::FunctionType FunctionType;
   typedef typename MapType::MatrixType MatrixType;
   typedef typename MatrixType::RowVectorType VectorType;
   typedef typename MatrixType::ScalarType ScalarType;
   typedef Norm<VectorType,MatrixType> NormType;

   Taylor(const MapType& Field, int Order, const ScalarType& Step);
   Taylor(const std::string& Field, int Order, const ScalarType& Step);
   Taylor(const char* Field, int Order, const ScalarType& Step);

   Taylor& operator=(const Taylor& a_t);
   VectorType Phi(const VectorType& iv) const;
   MatrixType JacPhi(const VectorType& iv) const;
   VectorType Remainder(const VectorType& iv) const;

   void JacRemainder(
         const VectorType &vecEnclosure,
         const MatrixType &jacEnclosure,
         VectorType &Remainder,
         MatrixType &jRemainder
      ) const;

   VectorType enclosure(const VectorType& x, int* found) const;
   VectorType enclosure(const VectorType &x) const;
   MatrixType jacEnclosure(const VectorType& enc, NormType& norm) const;

   using BasicTaylor<MapType>::getField;
   using BasicTaylor<MapType>::getOrder;
   using BasicTaylor<MapType>::getStep;
   using BasicTaylor<MapType>::getCoeff;
   using BasicTaylor<MapType>::getMatrixCoeff;

   using BasicTaylor<MapType>::setStep;
   using BasicTaylor<MapType>::setParameter;
   using BasicTaylor<MapType>::setOrder;
   using BasicTaylor<MapType>::setField;


protected:
   using BasicTaylor<MapType>::computeMatrixCoeff;
   using BasicTaylor<MapType>::m_vField;
   using BasicTaylor<MapType>::m_order;
   using BasicTaylor<MapType>::m_step;
   using BasicTaylor<MapType>::m_coeff;
   using BasicTaylor<MapType>::m_matrixCoeff;
   using BasicTaylor<MapType>::m_dimension;
   using BasicTaylor<MapType>::m_F;
};


// --------------- inline definitions -----------------

//###########################################################//

template<typename MapType>
inline typename Taylor<MapType>::VectorType Taylor<MapType>::Phi(const VectorType& iv) const
{
   return BasicTaylor<MapType>::operator()(iv);
}

template<typename MapType>
inline Taylor<MapType>& Taylor<MapType>::operator=(const Taylor& a_t)
{
   BasicTaylor<MapType>::operator=(a_t);
   return *this;
}

}} // namespace capd::dynsys

#endif // define _TaylorClass

/// @}
