/// @addtogroup normalForms
/// @{

/////////////////////////////////////////////////////////////////////////////
/// @file normalForms.h
///
/// @author Daniel Wilczak
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 2000-2005 by the CAPD Group.
//
// This file is part of the CAPD library.  This library is free software;
// you can redistribute it and/or modify it under the terms of the GNU
// General Public License as published by the Free Software Foundation;
// either version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this software; see the file "license.txt".  If not, write to the
// Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
// MA 02111-1307, USA.

#ifndef _normalForms_H
#define _normalForms_H

#include <complex>
#include <stdexcept>
#include <sstream>

#include "map/CnCoeff.h"
#include "vectalg/vector.h"
#include "vectalg/matrix.h"
#include "vectalg/multiindex.h"

namespace capd{
namespace normalForms{

// -------------------------------------------------------------------------- //

template<typename MatrixType>
void derivativesToSeries(capd::map::CnCoeff< MatrixType >& s)
{
   for(int i=2;i<=s.rank();++i)
   {
      typename capd::map::CnCoeff< MatrixType >::Multipointer mp = s.first(i);
      do{
         long fac = mp.factorial();
         for(int j=0;j<s.dimension();++j)
            s(j,mp)/=fac;
      }while(s.next(mp));
   }
}

}} // namespace capd::normalForms

#include "normalForms/linearSubstitution.hpp"
#include "normalForms/planarMaps.hpp"

#endif

/// @}
