/// @addtogroup 2
/// @{

/////////////////////////////////////////////////////////////////////////////
/// @file homInclAlgComplexity.cpp
///
/// @author Marian Mrozek
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) Marian Mrozek 2005-2006
//
// This file is part of the CAPD library.  This library is free software;
// you can redistribute it and/or modify it under the terms of the GNU
// General Public License as published by the Free Software Foundation;
// either version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this software; see the file "license.txt".  If not, write to the
// Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
// MA 02111-1307, USA.

#include <iostream>
#include <fstream>
#include <sstream>
#include <exception>
#include <new>
using namespace std;

#include "vectalg/matrixSlice.h"
#include "matrixAlgorithms/intMatrixAlgorithms.hpp"

#include "homologicalAlgebra/embeddingDim.h"
#include "auxil/ofstreamcout.h"
#include "auxil/Stopwatch.h"
#include "auxil/memSize.h"
#include "auxil/CRef.h"

extern ofstreamcout fcout;

#include "auxil/statistics.h"

using namespace capd;
using namespace matrixAlgorithms;
using namespace std;


#include "homologicalAlgebra/homologicalAlgebra.hpp"
#include "homologicalAlgebra/readCubFile.hpp"
#include "homologicalAlgebra/homAlgFunctors.hpp"
#include "homologicalAlgebra/cubSetFunctors.hpp"
#include "multiEngHom/homologyInterface.h"


/*
typedef CRef<HomologySignature> (*HomologyAlgorithm)(CRef<BCubSet>);
typedef CRef<HomologySignature> (*HomologyAlgorithmD2)(CRef<CubSetT<EuclBitSetT<BitSet,2> > >);
typedef CRef<HomologySignature> (*HomologyAlgorithmD3)(CRef<CubSetT<EuclBitSetT<BitSet,3> > >);
typedef CRef<HomologySignature> (*HomologyAlgorithmD4)(CRef<CubSetT<EuclBitSetT<BitSet,4> > >);
*/


typedef unsigned long int cluster;

#include "homologyComplexity/complexityTester.hpp"
#include "homologyComplexity/ObjectGenerators.h"
#include "auxil/handleUnexpectedTerminate.h"
#include "auxil/commandLineArgs.h"
//#include "multiEngHom/MultiEngHomT.h"
#include "bitSet/EmbDimException.h"
#include "homologyComplexity/SetPairT.h"
//#include "homologicalAlgebra/CrbInclHomology.hpp"

std::ofstream results;             // for results


template<int dim>
class HomAlgComplexity{
  private:
    int from,to,step;
    std::string inFileA,inFileX,alg,testType;
  public:
    HomAlgComplexity(
      int A_from,
      int A_to,
      int A_step,
      std::string A_inFileA,
      std::string A_inFileX,
      std::string A_alg,
      std::string A_testType
    ): from(A_from),to(A_to),step(A_step),
       inFileA(A_inFileA),inFileX(A_inFileX),
       alg(A_alg),testType(A_testType)
    {}
    typedef BitSetT<BitmapT<cluster> > BitSet;
    typedef EuclBitSetT<BitSet,dim> EuclBitSet;
    typedef CubSetT<EuclBitSet> BCubSet;
    typedef CubCellSetT<EuclBitSet> BCubCelSet;
    typedef CubCellSetT<EuclBitSetT<BitSetT<BitmapT<unsigned long int> >,dim> ,ReductionPairT<ECell3dCode> > SCubCelSet;
    typedef CRef<BCubSet> BCubSetCR;
    typedef CRef<BCubCelSet> BCubCelSetCR;
    typedef typename ComplexityTester<ObjectGenerator<SetPairT<BCubSet> >,std::string>::AlgorithmType HomologyInclusionAlgorithm;

    static CRef<std::string> CrbIncHom(CRef<SetPairT<BCubSet> > A_CubSetPairCR){
      CRef<SCubCelSet> subCubCelSetCR(new SCubCelSet((A_CubSetPairCR().first)()));
      CRef<SCubCelSet> supCubCelSetCR(new SCubCelSet((A_CubSetPairCR().second)()));
      return CRef<std::string>(new std::string(::CrbIncHom(subCubCelSetCR,supCubCelSetCR)));
    }
    static CRef<std::string> CrbIncHomViaHomPackage(CRef<SetPairT<BCubSet> > A_CubSetPairCR){
//      CRef<SCubCelSet> subCubCelSetCR(new SCubCelSet((A_CubSetPairCR().first)()));
//      CRef<SCubCelSet> supCubCelSetCR(new SCubCelSet((A_CubSetPairCR().second)()));
      return CRef<std::string>(new std::string(inclHomologyViaHomologyPackage<BCubSet>(A_CubSetPairCR().first,A_CubSetPairCR().second)));
    }
    void operator()(){
      std::string testDescription=testType;

      std::string s;
      if(testType == "inclusion"){
        CRef<BCubCelSet> aCubSetCR=readCubCellSet<BCubSet,BCubCelSet>(inFileA.c_str());
        CRef<BCubCelSet> xCubSetCR=readCubCellSet<BCubSet,BCubCelSet>(inFileX.c_str());
        CRef<ObjectGenerator<SetPairT<BCubSet> > > objectPairGeneratorCR(new RescaledPairObjectGenerator<BCubCelSet,BCubSet>(aCubSetCR,xCubSetCR));
//        HomologyInclusionAlgorithm selectedAlgorithm=MultiEngineHomology<dim>::selector.setupHomologyInclusionAlgorithm(alg);
        HomologyInclusionAlgorithm selectedAlgorithm;
        if(alg=="MM_CR"){
          selectedAlgorithm=CrbIncHom;
        }else if(alg=="PP"){
          selectedAlgorithm=CrbIncHomViaHomPackage;
        }else{
          throw std::string("Unknown algorithm");
        }
        ComplexityTester<ObjectGenerator<SetPairT<BCubSet> >,std::string> tester(
          *objectPairGeneratorCR,
          selectedAlgorithm,
          testDescription
        );
        s=tester.findComplexity(from,to,step,fcout);
      }
      fcout << s;
      results << s;
    }
};

int main(int argc,char** argv){
  try{
    set_unexpected(handle_unexpected);
    set_terminate(handle_terminate);
    fcout.open("details.out");
    results.open("summary.out");
    fcout.turnOn();
    readAcyclicConfigs();

    setupCommandLineArgs;
    declareCommandLineArg(int,from,1);
    declareCommandLineArg(int,to,0);
    declareCommandLineArg(int,step,1);
    declareCommandLineArg(std::string,inFileA,"");
    declareCommandLineArg(std::string,inFileX,"");
    declareCommandLineArg(std::string,alg,"MM_CR");
    declareCommandLineArg(std::string,testType,"scaled");
    declareCommandLineArg(std::string,help,"");
    if(!to) to=from;
    std::string executableName("homInclAlgComplexity");

    if((inFileX=="" && help=="") || help=="1"){
      help="intro";
    }

    if(help != ""){
      if(help=="intro"){
        fcout << "\n --- This is " << executableName << "  --- \n\n";
        fcout << "\n Author: M. Mrozek\n\n";
        fcout << "\n With homology software by W. Kalies,  M. Mrozek and P. Pilarczyk\n\n";
        fcout << "Usage: \n\n";
        fcout << "      " << executableName << " inFile=filename\n\n";
        fcout << "Type: \n\n";
        fcout << "      " << executableName << " help=options\n\n";
        fcout << "to learn more \n";
        exit(0);
      }else if(help=="options"){
        fcout << "\n --- This is " << executableName << "  --- \n\n";
        fcout << "To learn about available options type: \n\n";
        fcout << "      " << executableName << " help=xx\n\n";
        fcout << "where xx may be: \n\n";
        fcout << "tests - available tests\n";
        fcout << "algorithms - available algorithms\n";
        fcout << "input   - available input formats \n";
        exit(0);
      }else if(help=="algorithms"){
        fcout << "\n --- This is " << executableName << "  --- \n\n";
        fcout << "To select a particular homology algorithm implementation add option: \n\n";
        fcout << " alg=xx\n\n";
        fcout << "where xx denotes one of the available homology algorithms implementations: \n";
        fcout << "   BK or BK_LT\n";
        fcout << "     - written by Bill Kalies <wkalies@fau.edu>\n";
        fcout << "   MM_AS, MM_ASLT, MM_ASR, MM_ASH or MM_AR\n";
        fcout << "     - written by Marian Mrozek <Marian.Mrozek@ii.uj.edu.pl>\n";
        fcout << "   PP or PP_AS\n";
        fcout << "     - written by Pawel Pilarczyk <http://www.PawelPilarczyk.com/>\n";
        fcout << "You may add ns to the names of some implementations\n";
        fcout << "to prevent them from preprocessing the homology computations with shaving\n";
        exit(0);
      }else if(help=="input"){
        fcout << "\n --- This is " << executableName << "  --- \n\n";
        fcout << "" << executableName << " accepts files in the following formats\n";
        fcout << "   bmp - two dimensional bitmaps in the standard BMP bitmap format\n";
        fcout << "   bmd - multidimensional bitmaps in the bmd format\n";
        fcout << "   cub/txt - various text formats\n";
        fcout << "Consult http://www.math.gatech.edu/~chomp/ for details\n";
        fcout << "or see the enclosed examples\n";
        exit(0);
      }else if(help=="tests"){
        fcout << "\n --- This is " << executableName << "  --- \n\n";
        fcout << "The options controlling tests are  \n\n";
        fcout << "   from=m to=n step=k\n\n";
        fcout << "where \n";
        fcout << "   m - initial rescalling\n";
        fcout << "   n - final rescalling\n";
        fcout << "   k - step by which rescallings are increased\n";
        exit(0);
      }
    }
//    MultiEngineHomology<0> meh; // Needed to initialize engines!
//    HomAlgComplexity<2> hac2(from,to,step,inFileA,inFileX,alg,testType);
    HomAlgComplexity<3> hac3(from,to,step,inFileA,inFileX,alg,testType);
//    HomAlgComplexity<4> hac4(from,to,step,inFileA,inFileX,alg,testType);

    for(int i=2;i<5;++i){
      try{
        switch(i){
/*
          case 2: hac2();
                  goto done;
*/
          case 3: hac3();
                  goto done;
/*
          case 4: hac4();
                  goto done;
*/
          default:;
        }
      }catch(EmbDimException& e ){
        if(e.what() != std::string("Incorrect embDim")) throw;
      }
    }
    std::cout << "This executable supports only dimensions 2,3 and 4" << std::endl;
    done:;
  }catch(std::exception& e){
    fcout << "Caught exception: " << e.what() << endl;
  }catch(std::string& s){
    fcout << "Caught exception: " << s.c_str() << endl;
  }catch(const char* c){
    fcout << "Caught exception: " << c << endl;
  }catch(...){
    fcout << "Caught an unknown exception: " << endl;
  }
}
/// @}

