/// @addtogroup vectalg
/// @{

/////////////////////////////////////////////////////////////////////////////
/// @file imatrix.cpp
///
/// @author The CAPD Group
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 2000-2005 by the CAPD Group.
//
// This file is part of the CAPD library.  This library is free software;
// you can redistribute it and/or modify it under the terms of the GNU
// General Public License as published by the Free Software Foundation;
// either version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this software; see the file "license.txt".  If not, write to the
// Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
// MA 02111-1307, USA.


#include "interval/doubleInterval.h"
#include "vectalg/dimension.h"
#include "vectalg/vector.h"
#include "vectalg/matrix.h"

template class Matrix<interval,DIM,DIM>;
template class RowVector<interval,DIM>;
template class ColumnVector<interval,DIM>;

template Matrix<interval,DIM,DIM> abs <interval,DIM,DIM> (const Matrix<interval,DIM,DIM>&);
template Matrix<interval,DIM,DIM> operator- <interval,DIM,DIM> (const Matrix<interval,DIM,DIM>&);
template Matrix<interval,DIM,DIM> operator+ <interval,DIM,DIM> (const Matrix<interval,DIM,DIM>&, const Matrix<interval,DIM,DIM>&);
template Matrix<interval,DIM,DIM> operator- <interval,DIM,DIM> (const Matrix<interval,DIM,DIM>&, const Matrix<interval,DIM,DIM>&);
template Matrix<interval,DIM,DIM> operator* <interval,DIM,DIM> (const Matrix<interval,DIM,DIM>&, const Matrix<interval,DIM,DIM>&);
template Vector<interval,DIM> operator* <interval,DIM,DIM> (const Matrix<interval,DIM,DIM>&, const Vector<interval,DIM>&);
template Matrix<interval,DIM,DIM> operator* <interval,DIM,DIM> (const Matrix<interval,DIM,DIM>&, const interval&);
template Matrix<interval,DIM,DIM> operator* <interval,DIM,DIM> (const interval&, const Matrix<interval,DIM,DIM>&);
template Matrix<interval,DIM,DIM> operator/ <interval,DIM,DIM> (const Matrix<interval,DIM,DIM>&, const interval&);
template Matrix<interval,DIM,DIM> operator+ <interval,DIM,DIM> (const Matrix<interval,DIM,DIM>&, const interval&);
template Matrix<interval,DIM,DIM> operator- <interval,DIM,DIM> (const Matrix<interval,DIM,DIM>&, const interval&);
template bool operator< <interval,DIM,DIM> (const Matrix<interval,DIM,DIM>&, const Matrix<interval,DIM,DIM>&);
template bool operator> <interval,DIM,DIM> (const Matrix<interval,DIM,DIM>&, const Matrix<interval,DIM,DIM>&);
template bool operator<= <interval,DIM,DIM> (const Matrix<interval,DIM,DIM>&, const Matrix<interval,DIM,DIM>&);
template bool operator>= <interval,DIM,DIM> (const Matrix<interval,DIM,DIM>&, const Matrix<interval,DIM,DIM>&);
template Matrix<interval,DIM,DIM> Transpose <interval,DIM,DIM> (const Matrix<interval,DIM,DIM>&);
template std::ostream &operator<< <interval,DIM,DIM> (std::ostream&, const Matrix<interval,DIM,DIM>&);
template std::istream &operator>> <interval,DIM,DIM> (std::istream&, Matrix<interval,DIM,DIM>&);

// intervalMatrix

typedef Matrix<interval,DIM,DIM> IMatrix;
template void splitMatrix<IMatrix>(IMatrix&, IMatrix&);
template IMatrix midMatrix<IMatrix>(const IMatrix&);


// RowVector

template Vector<interval,DIM> operator+<interval,DIM>(
      const Vector<interval,DIM>&,
      const RowVector<interval,DIM>&
   );
template Vector<interval,DIM> operator+<interval,DIM>(
      const RowVector<interval,DIM>&,
      const Vector<interval,DIM>&
   );
template Vector<interval,DIM> operator+<interval,DIM>(
      const RowVector<interval,DIM>&,
      const RowVector<interval,DIM>&
   );

template Vector<interval,DIM> operator-<interval,DIM>(
      const Vector<interval,DIM>&,
      const RowVector<interval,DIM>&
   );
template Vector<interval,DIM> operator-<interval,DIM>(
      const RowVector<interval,DIM>&,
      const Vector<interval,DIM>&
   );
template Vector<interval,DIM> operator-<interval,DIM>(
      const RowVector<interval,DIM>&,
      const RowVector<interval,DIM>&
   );

template interval operator*<interval,DIM>(
      const Vector<interval,DIM>&,
      const RowVector<interval,DIM>&
   );
template interval operator*<interval,DIM>(
      const RowVector<interval,DIM>&,
      const Vector<interval,DIM>&
   );
template interval operator*<interval,DIM>(
      const RowVector<interval,DIM>&,
      const RowVector<interval,DIM>&
   );

template Vector<interval,DIM> operator-<interval,DIM>(const RowVector<interval,DIM>&);
template Vector<interval,DIM> operator*<interval,DIM,DIM>(
      const Matrix<interval,DIM,DIM>&, const RowVector<interval,DIM>&
   );

template Vector<interval,DIM> operator*<interval,DIM>(const interval&, const RowVector<interval,DIM>&);
template Vector<interval,DIM> operator*<interval,DIM>(const RowVector<interval,DIM>&, const interval&);
template Vector<interval,DIM> operator/<interval,DIM>(const RowVector<interval,DIM>&, const interval&);

template std::ostream& operator<< <interval,DIM>(std::ostream&, const RowVector<interval,DIM>&);


// ColumnVector

template Vector<interval,DIM> operator+<interval,DIM>(
      const Vector<interval,DIM>&,
      const ColumnVector<interval,DIM>&
   );
template Vector<interval,DIM> operator+<interval,DIM>(
      const ColumnVector<interval,DIM>&,
      const Vector<interval,DIM>&
   );
template Vector<interval,DIM> operator+<interval,DIM>(
      const ColumnVector<interval,DIM>&,
      const ColumnVector<interval,DIM>&
   );

template Vector<interval,DIM> operator-<interval,DIM>(
      const Vector<interval,DIM>&,
      const ColumnVector<interval,DIM>&
   );
template Vector<interval,DIM> operator-<interval,DIM>(
      const ColumnVector<interval,DIM>&,
      const Vector<interval,DIM>&
   );
template Vector<interval,DIM> operator-<interval,DIM>(
      const ColumnVector<interval,DIM>&,
      const ColumnVector<interval,DIM>&
   );

template interval operator*<interval,DIM>(
      const Vector<interval,DIM>&,
      const ColumnVector<interval,DIM>&
   );
template interval operator*<interval,DIM>(
      const ColumnVector<interval,DIM>&,
      const Vector<interval,DIM>&
   );
template interval operator*<interval,DIM>(
      const ColumnVector<interval,DIM>&,
      const ColumnVector<interval,DIM>&
   );

template Vector<interval,DIM> operator-<interval,DIM>(const ColumnVector<interval,DIM>&);
template Vector<interval,DIM> operator*<interval,DIM,DIM>(
      const Matrix<interval,DIM,DIM>&,
      const ColumnVector<interval,DIM>&
   );

template Vector<interval,DIM> operator*<interval,DIM>(const interval&, const ColumnVector<interval,DIM>&);
template Vector<interval,DIM> operator*<interval,DIM>(const ColumnVector<interval,DIM>&, const interval&);
template Vector<interval,DIM> operator/<interval,DIM>(const ColumnVector<interval,DIM>&, const interval&);

template std::ostream& operator<< <interval,DIM>(std::ostream&, const ColumnVector<interval,DIM>&);

/// @}
