/// @addtogroup setarth
/// @{

/////////////////////////////////////////////////////////////////////////////
/// @file lorenz4.cpp
///
/// @author The CAPD Group
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 2000-2005 by the CAPD Group.
//
// This file is part of the CAPD library.  This library is free software;
// you can redistribute it and/or modify it under the terms of the GNU
// General Public License as published by the Free Software Foundation;
// either version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this software; see the file "license.txt".  If not, write to the
// Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
// MA 02111-1307, USA.

#include "vectalg/vectalglib.h"
#include "lorenz4.h"

#if (DIM == 4)

intervalVector Lorenz4::f(const intervalVector &x) const
{
	intervalVector result;
	result[0] = s*(x[1]-x[0]);
	result[1] = (R-x[2])*x[0]-x[1];
	result[2] = x[0]*x[1]-q*x[2];
	result[3] = -L*x[3];
	return result;
}

intervalMatrix Lorenz4::df(const intervalVector &x) const
{
	intervalMatrix result;
	result[0][0] = -s;
	result[0][1] = s;
	result[0][2] = interval(0.);
	result[0][3] = interval(0.);
	result[1][0] = R-x[2];
	result[1][1] = interval(-1.);
	result[1][2] = -x[0];
	result[1][3] = interval(0.);
	result[2][0] = x[1];
	result[2][1] = x[0];
	result[2][2] = -q;
	result[2][3] = interval(0.);
	result[3][0] = interval(0.);
	result[3][1] = interval(0.);
	result[3][2] = interval(0.);
	result[3][3] = -L;
	return result;
}

intervalMatrix Lorenz4::d2f(int i, const intervalVector &x) const
{
	intervalMatrix result;
	if(i==1){
		result[0][2] = interval(-1.);
		result[2][0] = interval(-1.);
	}
	if(i==2){
		result[0][1] = interval(1.);
		result[1][0] = interval(1.);
	}
	return result;
}

#endif

/// @}
