/// @addtogroup setarth
/// @{

/////////////////////////////////////////////////////////////////////////////
/// @file lorenz4.h
///
/// @author The CAPD Group
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 2000-2005 by the CAPD Group.
//
// This file is part of the CAPD library.  This library is free software;
// you can redistribute it and/or modify it under the terms of the GNU
// General Public License as published by the Free Software Foundation;
// either version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this software; see the file "license.txt".  If not, write to the
// Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
// MA 02111-1307, USA.

#ifndef _lorenz_h_
#define _lorenz_h_
#include "capd/capdlib.h"

class Lorenz4 : public Ode{
public:
	Lorenz4(const interval& the_s, const interval& the_R, const interval& the_q, const interval& the_L);
	intervalVector f(const intervalVector &iv) const;
	intervalMatrix df(const intervalVector &iv) const;
	intervalMatrix d2f(int i, const intervalVector &iv) const;
	interval getR(void) const;
protected:
	interval s,R,q,L;
};



//inline definitions

inline Lorenz4::Lorenz4(const interval& the_s, const interval& the_R,
				const interval& the_q, const interval& the_L)
	: s(the_s),R(the_R),q(the_q),L(the_L)
{}

inline interval Lorenz4::getR(void) const
{
	return R;
}

#endif

/// @}
