/// @addtogroup vecttst
/// @{

/////////////////////////////////////////////////////////////////////////////
/// @file matrixMDTest.cpp
///
/// @author Marian Mrozek
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 2000-2005 by the CAPD Group.
//
// This file is part of the CAPD library.  This library is free software;
// you can redistribute it and/or modify it under the terms of the GNU
// General Public License as published by the Free Software Foundation;
// either version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this software; see the file "license.txt".  If not, write to the
// Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
// MA 02111-1307, USA.

#include <cassert>
#include <sstream>
#include <stdexcept>
#include "vectalg/vector.h"
#include "vectalg/matrix.h"

int main(int argc, char* argv[]){
   try{
      Matrix<double,0,0> m(1,1);
      std::istringstream s("{{5.0,4.5,4.0},{3.5,3.0,2.5},{-7.0 , 5, -8 }}");
      s >> m;
      std::cout << "m=" << m << std::endl;

      double data[]={
            1.0, 2.0, 3.0, 4.0,
            0.0, 1.0, 2.0, 3.0,
            0.0, 0.0, 1.0, 2.0
         };
      Matrix<double,0,0> A(3,4,data);
      std::cout << "A=" << A << std::endl;
      Matrix<double,0,0> B(4,3);
      B=Transpose(A);
      std::cout << "B=A^T=" << B << std::endl;
      Matrix<double,0,0> C(3,3);
      C=A*B;
      std::cout << "C=A*B=" <<  C << std::endl;
      Vector<double,0> v(1,2,3);
      std::cout << "C*"  << v << "=" << C*v << std::endl;
  }catch(std::exception& e){
    std::cout << "Exception caught: " << e.what() << std::endl;
  }catch(const char *c){
    std::cout << "Exception caught: " << c << std::endl;
  }catch(...){
    std::cout << "Unknown Exception caught: " << std::endl;
  }

  return 0;
}

/// @}
